#!/usr/bin/python3

# Mediadex: Index media metadata into opensearch
# Copyright (C) 2019-2022  Joni Harker
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.


import hashlib
import logging

import chardet
from pymediainfo import MediaInfo


class FileInfo:
    def __init__(self, f, b):
        self.log = logging.getLogger('mediadex.fileinfo')
        self.hashlib = hashlib.sha384()
        self.fullpath = f
        self.basepath = b
        self.mediainfo = None
        self.fingerprint = None

    def dumpData(self):
        output = {}
        output['fullpath'] = self.fullpath
        output['basepath'] = self.basepath
        output['mediainfo'] = self.mediainfo
        output['fingerprint'] = self.fingerprint

        return output

    def hashFile(self):
        # Magic numbers for file chunking
        chunk_size = 24576
        chunk_count = 128

        with open(self.fullpath, 'rb') as f:
            try:
                for _ in range(chunk_count):
                    chunk = f.read(chunk_size)
                    if chunk:
                        self.hashlib.update(chunk)
            except Exception as exc:
                if self.log.isEnabledFor(logging.INFO):
                    self.log.exception(exc)
                else:
                    self.log.warn(str(exc))

        self.fingerprint = self.hashlib.hexdigest()

    def parseMediaInfo(self):
        f = self.fullpath
        try:
            self.mediainfo = MediaInfo.parse(f).to_data()
        except FileNotFoundError:
            _enc = f.encode('utf-8', 'surrogateescape')
            charset = chardet.detect(f).get('encoding')
            self.log.info("chardet found: {}".format(charset))

            try:
                _f = _enc.decode(charset)
                self.mediainfo = MediaInfo.parse(_f).to_data()
            except FileNotFoundError:
                self.log.warning("chardet failure: {}".format(_f))

        except Exception as exc:
            if self.log.isEnabledFor(logging.INFO):
                self.log.exception(exc)
            else:
                self.log.warn(str(exc))

        finally:
            if not self.mediainfo:
                _f = f.encode('utf-8', 'surrogateescape')
                raise IOError("Could not open {}".format(_f))



#!/usr/bin/python3

# Mediadex: Index media metadata into opensearch
# Copyright (C) 2019-2022  K Jonathan Harker
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from opensearch_dsl import Document
from opensearch_dsl import Float
from opensearch_dsl import InnerDoc
from opensearch_dsl import Integer
from opensearch_dsl import Keyword
from opensearch_dsl import Long
from opensearch_dsl import Object
from opensearch_dsl import Text


class _Index:
    settings = {
        'number_of_shards': 1,
        'number_of_replicas': 0,
    }


class StreamCounts(InnerDoc):
    audio_stream_count: Integer()
    text_stream_count: Integer()
    video_stream_count: Integer()


class Media(Document):
    title = Text()
    year = Keyword()
    genre = Keyword(multi=True)
    stream_counts = Object(StreamCounts)
    container = Keyword()
    dirname = Keyword()
    filename = Keyword()
    filesize = Long()
    fingerprint = Text()


class Stream(InnerDoc):
    codec = Keyword()
    codec_profile = Keyword()
    duration = Float()
    language = Keyword()
    mime_type = Keyword()


class TextStream(Stream):
    charset = Keyword()


class BitStream(Stream):
    bit_rate = Long()
    bit_depth = Integer()
    sample_rate = Integer()


class AudioStream(BitStream):
    channels = Keyword()


class VideoStream(BitStream):
    resolution = Keyword()
    height = Integer()
    width = Integer()


class ID(InnerDoc):
    acoustid_fingerprint = Keyword()
    acoustid_id = Keyword()

    musicip_fingerprint = Keyword()
    musicip_puid = Keyword()


class Song(Media):
    audio_stream = Object(AudioStream)
    id_info = Object(ID)

    album = Text()
    albumartist = Text()
    arranger = Keyword()
    artist = Text()
    bpm = Float()
    compilation = Keyword()
    composer = Text()
    conductor = Text()
    discnumber = Keyword()
    mood = Keyword()
    performer = Text()
    tracknumber = Keyword()

    class Index(_Index):
        name = 'music'


class Cinema(Media):
    cast = Text(multi=True)
    director = Text(multi=True)
    writer = Text(multi=True)

    audio_streams = Object(AudioStream, multi=True)
    text_streams = Object(TextStream, multi=True)
    video_streams = Object(VideoStream, multi=True)


class Movie(Cinema):
    class Index(_Index):
        name = 'movies'


class Show(Cinema):
    season = Integer()

    class Index(_Index):
        name = 'series'



#!/usr/bin/python3

# Mediadex: Index media metadata into opensearch
# Copyright (C) 2019-2022  K Jonathan Harker
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import logging

from mutagen import MutagenError
from mutagen.easyid3 import EasyID3
from mutagen.id3._util import ID3NoHeaderError

from mediadex import ID
from mediadex import Song
from mediadex import StreamCounts

LOG = logging.getLogger('mediadex.indexer.song')


class SongIndexer:
    def __init__(self):
        pass

    def index(self, item, song=None):
        if song is None:
            song = Song()
        orig_dict = song.to_dict()

        song.audio_stream = next(item.astreams())
        song.dirname = item.dirname
        song.filename = item.filename
        song.filesize = item.general['file_size']
        song.fingerprint = item.fingerprint

        try:
            info = EasyID3(song.filename)
            id_doc = ID()

            if 'date' in info:
                song.year = info['date']

            for tag in ['album', 'albumartist', 'arranger', 'artist', 'bpm',
                        'compilation', 'composer', 'conductor', 'tracknumber',
                        'title', 'mood', 'genre', 'performer']:
                if tag in info:
                    setattr(song, tag, info[tag])

            for tag in ['musicip_puid', 'musicip_fingerprint',
                        'acoustid_fingerprint', 'acoustid_id']:
                if tag in info:
                    setattr(id_doc, tag, info[tag])
            song.id_info = id_doc

        except ID3NoHeaderError:
            pass
        except MutagenError as exc:
            if LOG.isEnabledFor(logging.INFO):
                LOG.exception(exc)
            else:
                LOG.warn(str(exc))

        stream_counts = StreamCounts()
        stream_counts.audio_stream_count = 1
        stream_counts.video_stream_count = 0
        stream_counts.text_stream_count = 0
        song.stream_counts = stream_counts

        if song.to_dict() != orig_dict:
            try:
                song.save()
            except Exception as exc:
                if LOG.isEnabledFor(logging.INFO):
                    LOG.exception(exc)
                else:
                    LOG.warn(str(exc))
            LOG.info("Song updated")
        else:
            LOG.debug("Song unchanged")



#!/usr/bin/python3

# Mediadex: Index media metadata into opensearch
# Copyright (C) 2019-2022  K Jonathan Harker
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import logging

from opensearch_dsl import FacetedSearch
from opensearch_dsl import TermsFacet

from mediadex import Movie
from mediadex import Song
from mediadex.exc import IndexerException
from mediadex.indexer.movie import MovieIndexer
from mediadex.indexer.song import SongIndexer
from mediadex.item import Item

LOG = logging.getLogger('mediadex.indexer')


class FileListFacet(FacetedSearch):
    fields = ['filename', ]
    facets = {'filenames': TermsFacet(field='filename')}

    def params(self, **kwargs):
        s = self._s._clone()
        r = s.params(**kwargs)
        self._s = r
        return r

    def execute(self):
        """
        Execute the search and return the response.
        """
        r = self._s.execute(ignore_cache=True)
        r._faceted_search = self
        return r

    def scan(self):
        for hit in self._s.scan():
            yield hit


class MovieListFacet(FileListFacet):
    doc_types = [Movie, ]

    def search(self):
        r = super().search().doc_type(Movie)
        return r.filter('exists', field='filename')


class SongListFacet(FileListFacet):
    doc_types = [Song, ]

    def search(self):
        r = super().search().doc_type(Song)
        return r.filter('exists', field='filename')


class Indexer:
    def __init__(self):
        Movie.init()
        Song.init()
        self.mi = MovieIndexer()
        self.si = SongIndexer()

    def index(self, info):
        LOG.debug('Hashing file')
        info.hashFile()

        LOG.debug('Calling mediainfo')
        info.parseMediaInfo()

        data = info.dumpData()
        item = Item(data['mediainfo']['tracks'])

        dirname = data['basepath'].rstrip('/')
        item.dirname = dirname

        filename = data['fullpath'].replace(dirname, '').lstrip('/')
        item.filename = filename

        item.fingerprint = data['fingerprint']

        if item.dex_type == 'empty':
            LOG.warn("No streams detected for {}".format(filename))
            return

        elif item.dex_type == 'unknown':
            LOG.warning('Unknown format ({}, {}, {}), '
                        'skipping {}'.format(len(item.video_tracks),
                                             len(item.audio_tracks),
                                             len(item.text_tracks),
                                             filename))

        elif item.dex_type == 'song':
            s = Song.search()
            LOG.info("Processing Song for {}".format(filename))
            r = s.filter('term', fingerprint=item.fingerprint).execute()

            if r.hits.total.value == 0:
                LOG.debug("Indexing new Song for {}".format(filename))
                self.index_song(item)
            elif r.hits.total.value == 1:
                LOG.debug("Updating existing Song for {}".format(filename))
                song = r.hits[0]
                self.index_song(item, song)
            else:
                LOG.error("Found {} existing Songs for {}".format(
                        r.hits.total.value, filename))
                for h in r.hits:
                    LOG.debug("{}/{}".format(h.dirname, h.filename))
                raise IndexerException("Multiple filename matches")

        elif item.dex_type == 'movie':
            s = Movie.search()
            LOG.info(f"Processing Movie for {filename} ({item.fingerprint})")
            r = s.filter('term', fingerprint=item.fingerprint).execute()

            if r.hits.total.value == 0:
                LOG.debug("Indexing new Movie for {}".format(filename))
                self.index_movie(item)
            elif r.hits.total.value == 1:
                LOG.debug("Updating existing Movie for {}".format(filename))
                movie = r.hits[0]
                self.index_movie(item, movie)
            else:
                LOG.error("Found {} existing Movies for {}".format(
                        r.hits.total.value, filename))
                for h in r.hits:
                    LOG.debug("{}/{}".format(h.dirname, h.filename))
                raise IndexerException("Multiple filename matches")

    def index_song(self, item, song=None):
        self.si.index(item, song)

    def index_movie(self, item, movie=None):
        self.mi.index(item, movie)



#!/usr/bin/python3

# Mediadex: Index media metadata into opensearch
# Copyright (C) 2019-2022  K Jonathan Harker
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import logging
import re

from imdb import Cinemagoer

from mediadex import Movie
from mediadex import StreamCounts

LOG = logging.getLogger('mediadex.indexer.movie')


class MovieIndexer:
    def __init__(self):
        self.imdb = Cinemagoer()

    def index(self, item, existing=None):
        movie = Movie()

        # basic stream info
        stream_counts = StreamCounts()

        vstreams = [x for x in item.vstreams()]
        tstreams = [x for x in item.tstreams()]
        astreams = [x for x in item.astreams()]

        movie.video_streams = vstreams
        movie.text_streams = tstreams
        movie.audio_streams = astreams

        stream_counts.video_stream_count = len(vstreams)
        LOG.debug("Processed {} video streams".format(len(vstreams)))
        stream_counts.text_stream_count = len(tstreams)
        LOG.debug("Processed {} text streams".format(len(tstreams)))
        stream_counts.audio_stream_count = len(astreams)
        LOG.debug("Processed {} audio streams".format(len(astreams)))

        movie.stream_counts = stream_counts
        movie.dirname = item.dirname
        movie.filename = item.filename
        movie.filesize = item.general['file_size']
        movie.fingerprint = item.fingerprint

        if existing:
            # Assume imdb hasn't changed anything
            movie.cast = existing.cast or None
            movie.director = existing.director or None
            movie.writer = existing.writer or None
            movie.title = existing.title or None
            movie.year = existing.year or None
            movie.genre = existing.genre or None

        else:
            # build a list of potential movie names
            # order matters
            #   title + subtitle (no year)
            #   title
            #   filename
            #   subtitle
            #   container movie_name
            #   container title
            search_strings = []

            file_name = item.general['file_name']
            file_sanitized = file_name.replace('.', ' ')

            file_title = None
            file_year = None
            file_subtitle = None
            file_re = re.compile(r'([^.]+ )+(\d{4})( [^.]*)*')
            file_match = file_re.match(file_sanitized)

            # check container metadata
            if 'movie_name' in item.general:
                search_strings.append(item.general['movie_name'])

            if 'title' in item.general:
                search_strings.append(item.general['title'])

            # parse re results
            if file_match:
                LOG.debug('filename parts: {}'.format(file_match.groups()))

                file_title = file_match.group(1).strip()
                file_year = file_match.group(2)
                if file_match.group(3):
                    file_subtitle = file_match.group(3).strip()

                LOG.debug("RE Match: {} {} {}".format(
                    file_title,
                    file_year,
                    file_subtitle)
                )

                if file_subtitle:
                    full_title = '{} {}'.format(file_title, file_subtitle)
                    search_strings.append(full_title)

                search_strings.append(file_title)

                if file_subtitle:
                    search_strings.append(file_subtitle)

            search_strings.append(file_sanitized)

            # collate search string results

            imdb_results = {}
            best_title = ''
            best_imdb = []
            imdb_info = {}

            LOG.debug(search_strings)
            for imdb_search in search_strings:
                LOG.debug("IMDB search: {}".format(imdb_search))
                if not imdb_search:
                    continue
                _imdb = self.imdb.search_movie(imdb_search)
                if _imdb:
                    imdb_results[imdb_search] = _imdb

            title_list = imdb_results.keys()
            for title in title_list:
                if not best_imdb:
                    best_title = title
                    best_imdb = imdb_results[title]

            found_count = len(title_list)
            if found_count == 0:
                LOG.warn("No IMDB match: {}".format(search_strings))
                LOG.debug(item.general)
                return
            elif found_count == 1:
                LOG.debug("One IMDB match: {}".format(best_title))
                imdb_info = best_imdb[0]
                LOG.debug(item.general)
            else:
                LOG.info("Best IMDB match: {}".format(best_title))
                imdb_info = best_imdb[0]
                LOG.debug(item.general)

            LOG.info("IMDB Title: {}".format(imdb_info['title']))
            self.imdb.update(imdb_info)

            try:
                if 'cast' in imdb_info:
                    movie.cast = [
                        x['name'] for x in imdb_info['cast'] if 'name' in x
                    ]
                if 'director' in imdb_info:
                    movie.director = [
                        x['name'] for x in imdb_info['director'] if 'name' in x
                    ]
                if 'writer' in imdb_info:
                    movie.writer = [
                        x['name'] for x in imdb_info['writer'] if 'name' in x
                    ]
                if 'title' in imdb_info:
                    movie.title = imdb_info['title']
                if 'year' in imdb_info:
                    movie.year = imdb_info['year']
                if 'genres' in imdb_info:
                    movie.genre = imdb_info['genres']
            except KeyError as exc:
                LOG.debug(imdb_info.__dict__)
                LOG.exception(exc)

        try:
            if existing is None:
                movie.save()
                LOG.info("Movie added")
            elif existing.to_dict() == movie.to_dict():
                LOG.debug("Movie unchanged")
            else:
                existing.delete()
                movie.save()
                LOG.info("Movie updated")

        except Exception as exc:
            if LOG.isEnabledFor(logging.INFO):
                LOG.exception(exc)
            else:
                LOG.warn(str(exc))



#!/usr/bin/python3

# Mediadex: Index media metadata into opensearch
# Copyright (C) 2019-2022  K Jonathan Harker
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import logging

from mediadex import AudioStream
from mediadex import TextStream
from mediadex import VideoStream
from mediadex.exc import IndexerException

LOG = logging.getLogger('mediadex.item')


class Item:
    def __init__(self, data):
        gen = [t for t in data if t['track_type'] == 'General']
        if len(gen) > 1:
            raise IndexerException("More than one General track found")
        elif len(gen) == 0:
            raise IndexerException("No General track found")
        self.general = gen.pop()

        atracks = [t for t in data if t['track_type'] == 'Audio']
        self.audio_tracks = atracks
        acount = len(atracks)

        vtracks = [t for t in data if t['track_type'] == 'Video']
        self.video_tracks = vtracks
        vcount = len(vtracks)

        ttracks = [t for t in data if t['track_type'] == 'Text']
        self.text_tracks = ttracks
        tcount = len(ttracks)

        if vcount > 0 and acount > 0:
            self.dex_type = 'movie'
        elif vcount > 0:
            self.dex_type = 'image'
        elif acount == 1:
            self.dex_type = 'song'
        elif tcount == 1:
            self.dex_type = 'text'
        elif acount == 0 and vcount == 0 and tcount == 0:
            self.dex_type = 'empty'
        else:
            self.dex_type = 'unknown'

    def base_track(self, track, stream):
        if 'duration' in track:
            try:
                stream.duration = float(track['duration'])
            except ValueError as exc:
                if LOG.isEnabledFor(logging.INFO):
                    LOG.exception(exc)
                else:
                    LOG.warn(str(exc))

        if 'format' in track:
            stream.codec = track['format']
        if 'format_profile' in track:
            stream.codec_profile = track['format_profile']
        if 'language' in track:
            stream.language = track['language']
        if 'internet_media_type' in track:
            stream.mime_type = track['internet_media_type']

    def astreams(self):
        for t in self.audio_tracks:
            stream = AudioStream()

            self.base_track(t, stream)

            if 'channel_s' in t:
                stream.channels = t['channel_s']

            if 'bit_rate' in t:
                if t['bit_rate'] == 'Open':
                    continue

                try:
                    stream.bit_rate = int(t['bit_rate'])
                except ValueError as verr:
                    LOG.warn(str(verr))
                except Exception as exc:
                    LOG.exception(exc)

            if 'sampling_rate' in t:
                stream.sample_rate = t['sampling_rate']

            yield stream

    def tstreams(self):
        for t in self.text_tracks:
            stream = TextStream()

            self.base_track(t, stream)

            if 'encoding' in t:
                stream.charset = t['encoding']

            yield stream

    def vstreams(self):
        for t in self.video_tracks:
            stream = VideoStream()

            self.base_track(t, stream)

            if 'bit_rate' in t:
                if t['bit_rate'] == 'Open':
                    continue

                try:
                    stream.bit_rate = int(t['bit_rate'])
                except ValueError as verr:
                    LOG.warn(str(verr))
                except Exception as exc:
                    LOG.exception(exc)

            if 'bit_depth' in t:
                try:
                    stream.bit_depth = int(t['bit_depth'])
                except Exception as exc:
                    if LOG.isEnabledFor(logging.INFO):
                        LOG.exception(exc)
                    else:
                        LOG.warn(str(exc))

            if 'height' in t and 'width' in t:
                try:
                    stream.height = int(t['height'])
                    stream.width = int(t['width'])

                    stream.resolution = "{0}x{1}".format(
                            t['height'],
                            t['width'],
                    )
                except ValueError as exc:
                    if LOG.isEnabledFor(logging.INFO):
                        LOG.exception(exc)
                    else:
                        LOG.warn(str(exc))

            yield stream



#!/usr/bin/python3

# Mediadex: Index media metadata into opensearch
# Copyright (C) 2019-2022  Joni Harker
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.


import argparse
import datetime
import logging
import os

import yaml
from opensearch_dsl import connections

from mediadex.cmd.fileinfo import FileInfo
from mediadex.indexer import Indexer
from mediadex.indexer import IndexerException
from mediadex.purger import MoviePurger
from mediadex.purger import SongPurger


class App:
    def __init__(self):
        self.args = None
        self.dex = None
        self.log = None
        self.client = None

    def parse_args(self):
        parser = argparse.ArgumentParser()

        parser.add_argument('-p', '--path',
                            dest='path',
                            action='append',
                            help='top directory to search for media')

        parser.add_argument('-v', '--verbose',
                            dest='verbose',
                            action='count',
                            help='output additional log messages')

        parser.add_argument('--purge',
                            dest='purge',
                            action='store_true',
                            help='Scan for deleted files and remove '
                            'their entries from opensearch')

        parser.add_argument('--today',
                            dest='today',
                            action='store_true',
                            help='only scan recent files')

        parser.add_argument('-H', '--opensearch-host',
                            dest='host',
                            action='store', default='localhost:9200',
                            help='opensearch host to connect to')

        parser.add_argument('--opensearch-user-pass',
                            dest='userpass',
                            action='store', default='admin:admin',
                            help='opensearch host to connect to')

        parser.add_argument('--insecure',
                            dest='insecure',
                            action='store_true',
                            help='ignore tls failures')

        parser.add_argument('-x', '--dry-run',
                            dest='dry_run',
                            action='store_true',
                            help='write to stdout as yaml instead of '
                            'indexing into opensearch')

        self.args = parser.parse_args()

    def setup_logging(self, level):
        root_log = logging.getLogger()
        root_log.setLevel(level)
        sh = logging.StreamHandler()
        sh.setLevel(level)
        fmt = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
        sh.setFormatter(logging.Formatter(fmt))
        root_log.addHandler(sh)

        for lib in ['opensearch', 'imdb', 'imdbpy', 'urllib3']:
            log = logging.getLogger(lib)
            log.setLevel(logging.WARNING)

        self.log = logging.getLogger('mediadex')
        self.log.setLevel(level)

    def index(self, fi):
        if self.args.dry_run:
            self.log.info(yaml.dump(fi))
        else:
            try:
                self.dex.index(fi)
            except IndexerException as exc:
                if self.log.isEnabledFor(logging.INFO):
                    self.log.exception(exc)
                else:
                    self.log.warn(str(exc))
                self.log.debug(yaml.dump(fi))
                return 1
        return 0

    def open_file(self, fp, bp):
        info = FileInfo(fp, bp)

        if self.args.today:
            _stat = os.stat(fp)
            mtime = datetime.datetime.fromtimestamp(_stat.st_mtime)
            diff = datetime.datetime.now() - mtime
            if diff > datetime.timedelta(days=1):
                self.log.debug('Skipping {} due to timestamp'.format(fp))
                return 0

        return self.index(info)

    def walk_paths(self):
        retval = 0
        for path in self.args.path:
            for (_top, _dirs, _files) in os.walk(path):
                for _file in _files:
                    fp = os.path.join(_top, _file)
                    try:
                        retval += self.open_file(fp, path)
                    except Exception as exc:
                        if self.log.isEnabledFor(logging.INFO):
                            self.log.exception(exc)
                        else:
                            self.log.warn(str(exc))
                        retval += 1
        return retval

    def purge(self):
        retval = 0
        try:
            sp = SongPurger()
            mp = MoviePurger()
            retval += sp.purge() + mp.purge()
        except Exception as exc:
            self.log.exception(exc)
            retval += 1
        return retval

    def run(self):
        self.parse_args()

        if self.args.verbose is None:
            self.setup_logging(level=logging.WARNING)
        elif self.args.verbose == 1:
            self.setup_logging(level=logging.INFO)
        else:
            self.setup_logging(level=logging.DEBUG)

        if not self.args.dry_run:
            host, port = self.args.host.split(':')
            user, pw = self.args.userpass.split(':')

            secure = True
            if self.args.insecure:
                import urllib3
                urllib3.disable_warnings()
                secure = False

            connections.create_connection(
              hosts=[{'host': host, 'port': port}],
              http_auth=(user, pw),
              use_ssl=True,
              verify_certs=secure,
              ssl_assert_hostname=secure,
            )

            self.dex = Indexer()
            if self.args.purge:
                return self.purge()

        return self.walk_paths()



#!/usr/bin/python3

# Mediadex: Index media metadata into opensearch
# Copyright (C) 2019-2022  K Jonathan Harker
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import logging
import os.path

from mediadex import Movie
from mediadex import Song

LOG = logging.getLogger('mediadex.purger')


class Purger:
    baseQ = None

    def purge(self):
        retval = 0

        for h in self.baseQ.scan():
            fullpath = os.path.join(h.dirname, h.filename)
            if not os.path.exists(fullpath):
                try:
                    _id = h.meta._d_['id']
                    LOG.warn('Purging {} for {}'.format(_id, fullpath))
                    h.delete()
                except Exception as exc:
                    LOG.exception(exc)
                    retval += 1

        return retval


class MoviePurger(Purger):
    def __init__(self):
        self.baseQ = Movie.search().filter('exists', field='filename')


class SongPurger(Purger):
    def __init__(self):
        self.baseQ = Song.search().filter('exists', field='filename')
